@@ -20,26 +20,12 @@
 #  part of this work is illegal and unethical regarding the effort and
 #  time spent here.
 #
-#  ldict is free software: you can redistribute it and/or modify
-#  it under the terms of the GNU General Public License as published by
-#  the Free Software Foundation, either version 3 of the License, or
-#  (at your option) any later version.
-#
-#  ldict is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with ldict.  If not, see <http://www.gnu.org/licenses/>.
-#
-#  (*) Removing authorship by any means, e.g. by distribution of derived
-#  works or verbatim, obfuscated, compiled or rewritten versions of any
-#  part of this work is a crime and is unethical regarding the effort and
-#  time spent here.
 import json
+import operator
 from collections import UserDict
 from copy import deepcopy
+from functools import reduce
+from random import Random
 from typing import Dict, TypeVar, Union, Callable
 
 from orjson import dumps, OPT_SORT_KEYS
@@ -50,6 +36,7 @@ from ldict.apply import delete, application
 from ldict.customjson import CustomJSONEncoder
 from ldict.data import key2id
 from ldict.exception import MissingField, ReadOnlyLdict, WrongKeyType, WrongValueType, OverwriteException
+from ldict.functionspace import FunctionSpace
 from ldict.history import extend_history, rewrite_history
 from ldict.lazy import Lazy
 
@@ -160,9 +147,10 @@ class Ldict(UserDict, Dict[str, VT]):
         self.delete = identity.delete
         self.history = {}
         self.last = None
+        self.rnd = Random()
         super().__init__()
         self.data.update(id=identity.id, ids={})
-        self.update(**kwargs)
+        self.update(**(_dictionary or {}), **kwargs)
 
     def __setitem__(self, key: str, value):
         if self.readonly:
@@ -171,7 +159,7 @@ class Ldict(UserDict, Dict[str, VT]):
             raise WrongKeyType(f"Key must be string, not {type(key)}.", key)
         if callable(value):
             raise WrongValueType(f"A value for the field [{key}] cannot have type {type(value)}. "
-                            f"For (pseudo)inplace function application, use operator >>= instead")
+                                 f"For (pseudo)inplace function application, use operator >>= instead")
         if isinstance(value, Ldict):
             value = value.clone(readonly=True)
 
@@ -280,8 +268,11 @@ class Ldict(UserDict, Dict[str, VT]):
         """
         return self.__repr__(all=True)
 
-    def __rshift__(self, other: Union[Dict, Callable]):
+    def __rshift__(self, other: Union[Dict, Callable, FunctionSpace], config={}, rnd=None):
+        from ldict.cfg import cfg
         clone = self.clone()
+        if rnd is not None:
+            clone.rnd = Random(rnd) if isinstance(rnd, int) else rnd
 
         # Insertion of dict-like.
         if isinstance(other, Dict):
@@ -295,10 +286,18 @@ class Ldict(UserDict, Dict[str, VT]):
                         raise OverwriteException(f"Cannot overwrite field ({k}) via value insertion through >>")
                     clone[k] = v
             return clone
+        elif isinstance(other, FunctionSpace):
+            return reduce(operator.rshift, (clone,) + other.functions)
+        elif isinstance(other, cfg):
+            from ldict.cfg import Ldict_cfg
+            d = Ldict_cfg(clone, other.config, other.rnd)
+            if other.f:
+                d >>= other.f
+            return d
         elif not callable(other):
             raise WrongValueType(f"Value passed to >> should be callable or dict-like, not {type(other)}")
 
-        return application(self, clone, other)
+        return application(self, clone, other, config, clone.rnd)
 
     def clone(self, readonly=False):
         """
@@ -326,6 +325,7 @@ class Ldict(UserDict, Dict[str, VT]):
         obj.data["ids"] = self.data["ids"].copy()
         obj.history = deepcopy(self.history)
         obj.last = self.last
+        obj.rnd = self.rnd  # Do not clone rnd generator. Ldict will be cloned
         return obj
 
     def evaluate(self):
@@ -384,6 +384,7 @@ class Ldict(UserDict, Dict[str, VT]):
 
         return closure
 
+    # TODO remover referencia a self do closure
     def __xor__(self, f: Union[Dict, Callable]):
         def closure(self, output_field):
             if output_field in self.hashes:
@@ -443,12 +444,15 @@ class Ldict(UserDict, Dict[str, VT]):
                 dic["ids"] = " ".join(self.ids.values())
             else:
                 ids = list(self.ids.values())
-                dic["ids"] = f"{ids[0]}... +{(len(self) - 1) // 2} ...{ids[-1]}"
+                dic["ids"] = f"{ids[0]}... +{len(self) - 4} ...{ids[-1]}"
         return json.dumps(dic, indent=4, ensure_ascii=False, cls=CustomJSONEncoder)
 
     @property
     def asdict(self):
         return ldict2dic(self, all=True)
 
+    def __mul__(self, other):
+        return FunctionSpace(other)
+
 
 ldict = Ldict
